unit Unit1;

{-$DEFINE CB_FUNC}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, MMSystem, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    cmbDevice: TComboBox;
    btnStart: TButton;
    btnStop: TButton;
    btnReset: TButton;
    pbNow: TProgressBar;
    pbMax: TProgressBar;
    tmrBars: TTimer;
    lblNow: TLabel;
    lblMax: TLabel;
    lblNowText: TLabel;
    lblMaxText: TLabel;
    btnGetDevList: TButton;
    lblBufsText: TLabel;
    edtBufs: TEdit;
    btnTest: TButton;
    lblMisc: TLabel;
    btnPause: TButton;
    btnClose: TButton;
    chkLog: TCheckBox;
    chkLarge: TCheckBox;
    function ProcessError(Err: Integer): Boolean;
{$IFNDEF CB_FUNC}
    procedure waveMsgProc(var Msg: TMessage); message MM_WIM_DATA;
{$ENDIF}
    function GetDevList: Integer;
    procedure FormCreate(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure StopRec(Reset: Boolean);
    procedure btnStopClick(Sender: TObject);
    procedure tmrBarsTimer(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure btnGetDevListClick(Sender: TObject);
    procedure btnTestClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure chkLargeClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type
  TBuf = array [0..44100] of SmallInt;
  PBuf = ^TBuf;
const
  BufN = 10;
var
  WaveHdr: array [0..BufN-1] of TWAVEHDR;
  WaveH: HWAVEIN;
  Buf: array [0..BufN-1] of TBuf;
  ErrMsg: array [0..255] of Char;
  WaveNow, WaveMax, Bufs, CurBuf: Integer;
  SD: Integer = 10;
  Stop, Done: Boolean;

function TForm1.ProcessError(Err: Integer): Boolean;
var i: Integer;
begin
  Result := (Err <> MMSYSERR_NOERROR);
  if not Result then Exit;
  if WaveH <> 0 then
  begin
    waveInReset(WaveH);
    waveInClose(WaveH);
    for i := 0 to BufN-1 do
      if WaveHdr[i].dwFlags and WHDR_PREPARED > 0 then waveInUnprepareHeader(WaveH, @WaveHdr[i], SizeOf(WaveHdr[i]));
    WaveH := 0
  end;
  waveInGetErrorText(Err, @ErrMsg, SizeOf(ErrMsg));
  MessageBox(0, ErrMsg, PChar(' '+IntToStr(Err)), MB_OK or MB_ICONERROR or MB_TASKMODAL);
  btnStart.Enabled := True;
  btnPause.Enabled := False;
  btnPause.Caption := '';
  btnStop.Enabled := False;
  btnClose.Enabled := False;
  cmbDevice.Enabled := True;
  btnGetDevList.Enabled := True;
end;

{$IFNDEF CB_FUNC}
procedure TForm1.waveMsgProc(var Msg: TMessage);
var i, N, wMin, wMax: Integer;
begin
  if WaveH = 0 then Exit;
  with PWAVEHDR(Msg.lParam)^ do
  begin
    WaveNow := 0;
    wMin := 0;
    wMax := 0;
    for i := 0 to dwBytesRecorded div 2-1 do
    begin
      N := PBuf(lpData)^[i];
      if N < wMin then wMin := N;
      if N > wMax then wMax := N
    end;
    N := (wMax - wMin) div 2;
    WaveNow := N;
    if N > WaveMax then WaveMax := N;
    if not Stop then
    begin
      lblMisc.Caption := 'Buf=' + IntToStr(CurBuf) + ', DONE=' + IntToStr(Integer(dwFlags and WHDR_DONE));
      dwFlags := dwFlags and (not WHDR_DONE);
      dwBytesRecorded := 0;
      dwBufferLength := SizeOf(Buf[0]) div SD;
      ProcessError(waveInAddBuffer(WaveH, PWAVEHDR(Msg.lParam), SizeOf(WaveHdr[0])));
      CurBuf := (CurBuf+1) mod BufN
    end
  end;
  Inc(Bufs)
end;
{$ELSE}
procedure waveInProc(hwi: HWAVEIN; uMsg, dwInstance, dwParam1, dwParam2: DWord); stdcall;
var i, N, wMin, wMax: Integer;
begin
  if uMsg = WIM_DATA then
  begin
    Inc(Bufs);
    with PWAVEHDR(dwParam1)^ do
    begin
      WaveNow := 0;
      wMin := 0;
      wMax := 0;
      for i := 0 to dwBytesRecorded div 2-1 do
      begin
        N := PBuf(lpData)^[i];
        if N < wMin then wMin := N;
        if N > wMax then wMax := N
      end;
      N := (wMax - wMin) div 2;
      WaveNow := N;
      if N > WaveMax then WaveMax := N;
      if not Stop then
      begin
        if WaveHdr[dwUser].dwFlags and WHDR_DONE <> 0 then Done := True;
        dwFlags := dwFlags and (not WHDR_DONE);
        dwBytesRecorded := 0;
        dwBufferLength := SizeOf(Buf[0]) div SD;
        waveInAddBuffer(WaveH, PWAVEHDR(dwParam1), SizeOf(WaveHdr[0]));
        CurBuf := (CurBuf+1) mod BufN
      end
    end
  end
end;
{$ENDIF}

//          -2,    
function TForm1.GetDevList: Integer;
var
  i, N: Integer;
  DevIn: TWAVEINCAPS;
  S: String;
begin
  Result := -2;
  S := cmbDevice.Text;
  cmbDevice.Clear;
  N := 0;
  for i := -1 to waveInGetNumDevs-1 do
  begin
    FillChar(DevIn, SizeOf(DevIn), 0);
    if waveInGetDevCaps(i, @DevIn, SizeOf(DevIn)) = MMSYSERR_NOERROR then cmbDevice.Items.Add(DevIn.szPname);
    if DevIn.szPname = S then
    begin
      N := cmbDevice.Items.Count-1;
      Result := i
    end
  end;
  cmbDevice.ItemIndex := N
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  GetDevList
end;

procedure TForm1.btnStartClick(Sender: TObject);
var
  WaveFmt: TWAVEFORMATEX;
  i, DevID: Integer;
begin
  edtBufs.Text := '0';
  DevID := GetDevList;
  if DevID = -2 then
    if MessageBox(0, PChar('   .'#13'     ?'), '', MB_YESNO or MB_ICONWARNING or MB_TASKMODAL) = idNo then Exit
    else DevID := -1;
  btnStart.Enabled := False;
  cmbDevice.Enabled := False;
  btnGetDevList.Enabled := False;

  with WaveFmt do
  begin
    wFormatTag := WAVE_FORMAT_PCM;
    nChannels := 1;
    nSamplesPerSec := 44100;
    wBitsPerSample := 16;
    nBlockAlign := nChannels*wBitsPerSample shr 3;
    nAvgBytesPerSec := nSamplesPerSec*nBlockAlign;
    cbSize := 0
  end;
{$IFDEF CB_FUNC}
  if ProcessError(waveInOpen(@WaveH, DevID, @WaveFmt, DWord(@waveInProc), 0, CALLBACK_FUNCTION)) then Exit;
{$ELSE}
  if ProcessError(waveInOpen(@WaveH, DevID, @WaveFmt, DWord(Form1.Handle), 0, CALLBACK_WINDOW)) then Exit;
{$ENDIF}

  CurBuf := 0;
  FillChar(WaveHdr, SizeOf(WaveHdr), 0);
  for i := 0 to BufN-1 do
  begin
    with WaveHdr[i] do
    begin
      lpData := @Buf[i];
      dwBufferLength := SizeOf(Buf[i]) div SD;
      dwUser := i
    end;
    if ProcessError(waveInPrepareHeader(WaveH, @WaveHdr[i], SizeOf(WaveHdr[i]))) then Exit;
    if ProcessError(waveInAddBuffer(WaveH, @WaveHdr[i], SizeOf(WaveHdr[i]))) then Exit
  end;

  WaveNow := 0;
  WaveMax := 0;
  Bufs := 0;
  Stop := False;
  if ProcessError(waveInStart(WaveH)) then Exit;

  btnPause.Enabled := True;
  btnStop.Enabled := True;
  btnClose.Enabled := True;
  tmrBars.Enabled := True
end;

procedure TForm1.StopRec;
var i, Err: Integer;
begin
  btnPause.Enabled := False;
  btnPause.Caption := '';
  btnStop.Enabled := False;
  btnClose.Enabled := False;

  Stop := True;
  if Reset then waveInReset(WaveH);

  Err := waveInClose(WaveH);
  if Err <> MMSYSERR_NOERROR then
  begin
    waveInGetErrorText(Err, @ErrMsg, SizeOf(ErrMsg));
    MessageBox(0, ErrMsg, PChar(' '+IntToStr(Err)), MB_OK or MB_ICONERROR or MB_TASKMODAL);
  end;

  for i := 0 to BufN-1 do
    waveInUnprepareHeader(WaveH, @WaveHdr[i], SizeOf(WaveHdr[i]));

  tmrBarsTimer(nil);
  tmrBars.Enabled := False;
  GetDevList;
  cmbDevice.Enabled := True;
  btnGetDevList.Enabled := True;
  btnStart.Enabled := True;
  WaveH := 0
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
  StopRec(True)
end;

procedure TForm1.btnResetClick(Sender: TObject);
begin
  WaveNow := 0;
  WaveMax := 0;
  tmrBarsTimer(nil)
end;

procedure TForm1.tmrBarsTimer(Sender: TObject);
begin
  if chkLog.Checked then
  begin
    pbNow.Position := Round(Ln(WaveNow+1)/Ln(MAXSHORT+1)*100);
    if WaveNow = 0 then lblNow.Caption := '-Inf db'
    else lblNow.Caption := IntToStr(Round((Ln(WaveNow)-Ln(MAXSHORT))/Ln(10)*20)) + ' db';
    pbMax.Position := Round(Ln(WaveMax+1)/Ln(MAXSHORT+1)*100);
    if WaveMax = 0 then lblMax.Caption := '-Inf db'
    else lblMax.Caption := IntToStr(Round((Ln(WaveMax)-Ln(MAXSHORT))/Ln(10)*20)) + ' db'
  end
  else
  begin
    pbNow.Position := Round(WaveNow/MAXSHORT*100);
    if WaveNow = 0 then lblNow.Caption := '-Inf db'
    else lblNow.Caption := IntToStr(WaveNow);
    pbMax.Position := Round(WaveMax/MAXSHORT*100);
    if WaveMax = 0 then lblMax.Caption := '-Inf db'
    else lblMax.Caption := IntToStr(WaveMax)
  end;
  edtBufs.Text := IntToStr(Bufs);
  if not Done then Done := Boolean(WaveHdr[0].dwFlags and WHDR_DONE);
  lblMisc.Caption := 'Buf=' + IntToStr(CurBuf) + ', DONE=' + IntToStr(Integer(Done));
  Done := False
end;

procedure TForm1.btnGetDevListClick(Sender: TObject);
begin
  GetDevList
end;

procedure TForm1.btnTestClick(Sender: TObject);
const
  DRVM_MAPPER_PREFERRED_GET = $2000+21;
  DRVM_MAPPER_PREFERRED_SET = $2000+22;
var
  DevIn: TWAVEINCAPS;
  ID, Flags, Err: DWord;
begin
  Err := waveInMessage(WAVE_MAPPER, DRVM_MAPPER_PREFERRED_GET, DWord(@ID), DWord(@Flags));
  if Err = MMSYSERR_NOERROR then
  begin
    FillChar(DevIn, SizeOf(DevIn), 0);
    if waveInGetDevCaps(ID, @DevIn, SizeOf(DevIn)) = MMSYSERR_NOERROR then
      MessageBox(0, DevIn.szPname, PChar(IntToStr(ID) + ' / ' + IntToStr(Flags)), MB_OK or MB_ICONINFORMATION or MB_TASKMODAL)
    else
    begin
      waveInGetErrorText(Err, @ErrMsg, SizeOf(ErrMsg));
      MessageBox(0, PChar(' '+IntToStr(Err)+': '+ErrMsg), PChar(IntToStr(ID) + ' / ' + IntToStr(Flags)), MB_OK or MB_ICONINFORMATION or MB_TASKMODAL)
    end
  end
  else
  begin
    waveInGetErrorText(Err, @ErrMsg, SizeOf(ErrMsg));
    MessageBox(0, ErrMsg, PChar(' '+IntToStr(Err)), MB_OK or MB_ICONERROR or MB_TASKMODAL)
  end
end;

procedure TForm1.btnPauseClick(Sender: TObject);
begin
  if btnPause.Caption = '' then
  begin
    waveInStop(WaveH);
    btnPause.Caption := ''
  end
  else
  begin
    waveInStart(WaveH);
    btnPause.Caption := ''
  end;
end;

procedure TForm1.btnCloseClick(Sender: TObject);
begin
  StopRec(False)
end;

procedure TForm1.chkLargeClick(Sender: TObject);
begin
  if chkLarge.Checked then SD := 1 else SD := 10
end;

end.
